#include <Wire.h>
#include "ACROBOTIC_SSD1306.h"
#include "DFRobot_SHT20.h"

DFRobot_SHT20 sht20;

// constants

const int warmModeSSRPin = 5;           // SSR2 
const int cookModeSSRPin = 4;           // SSR1 
const float targetedHumidity = 80;      // targeted level of humidity
const float targetedTemperature = 24;   // targeted temperature
const int ramp = 1000;                  // initial ramp (in milliseconds)
const float tempMargin = 0.5;           // lower and upper margin for temperature (in Celcius). As far as the current temp remains between 
                                        // (target - tempMargin) and (target + tempMArgin), no action is taken 
const float humMargin = 10;             // lower and upper margin for humidity (in %). As far as the current humidity remains between 
                                        // (target - humMargin) and (target + humMArgin), no action is taken 

// variables

float currentHumidity = 0;              // current level of humidity in the proofing box (in %)
float currentTemperature = 0;           // current temperature (in Celcius)

// This is the setup routine. It runs once, at power up 

void setup() {
  // Temperature and humidity sensor
  sht20.initSHT20();                     
  delay(100);
  sht20.checkSHT20(); 
  Serial.begin(9600);  
  // OLED screen
  oled.init();
  // Relays
  pinMode(cookModeSSRPin, OUTPUT); 
  digitalWrite(cookModeSSRPin, LOW);
  pinMode(warmModeSSRPin, OUTPUT); 
  digitalWrite(warmModeSSRPin, LOW);
  //Serial.println(F("End of setup"));
}

// This is the main loop, it runs forever

void loop() {
  getDataFromProofingBox();
  updateDisplay();
  publish();
  if (millis() < ramp || currentHumidity < targetedHumidity - humMargin){
    digitalWrite(warmModeSSRPin, HIGH);
    digitalWrite(cookModeSSRPin, HIGH);
  }
  else{
    digitalWrite(cookModeSSRPin, LOW);
    if (currentTemperature < targetedTemperature - tempMargin ){
      digitalWrite(warmModeSSRPin, HIGH);
    }    
    else{
      if (currentTemperature > targetedTemperature + tempMargin ){
         digitalWrite(warmModeSSRPin, LOW);
      }
      else{
        // no change, keep warming to maintain optimal conditions
      }
    }
  }
  delay(5000);
}

// Get humidity and temperature measurements from the proofing box

void getDataFromProofingBox(){
  currentHumidity = sht20.readHumidity();             // Read Humidity
  currentTemperature = sht20.readTemperature();       // Read Temperature
}

// Display data on the OLED screen

void updateDisplay(){
  char c[8];
  String s;
  String line;
  oled.clearDisplay(); 
  oled.setFont(font8x8);   
  // Line 1
  line = "Temp (C) = %1"; 
  dtostrf(currentTemperature, 2, 1, c);
  s = "";
  for (int i=0; i<sizeof(c); i++){
    s += c[i];
  }
  line.replace("%1",s);
  oled.setTextXY(1,0); 
  oled.putString(line);
  
  // Line 2
  line = "Target (C) = %1"; 
  dtostrf(targetedTemperature, 2, 0, c);
  s = "";
  for (int i=0; i<sizeof(c); i++){
    s += c[i];
  }
  line.replace("%1",s);
  oled.setTextXY(2,0); 
  oled.putString(line);
  
  // Line 3
  line = "Hum (%) = %1";  
  dtostrf(currentHumidity, 2, 1, c);
  s = "";
  for (int i=0; i<sizeof(c); i++){
    s += c[i];
  }
  line.replace("%1",s);
  oled.setTextXY(3,0); 
  oled.putString(line);
  
  // Line 4
  line = "Target (%) = %1";  
  dtostrf(targetedHumidity, 2, 0, c);
  s = "";
  for (int i=0; i<sizeof(c); i++){
    s += c[i];
  }
  line.replace("%1",s);
  oled.setTextXY(4,0); 
  oled.putString(line);
  
  // Line 5
  line = "Time %1 min";  
  int minutes = millis()/1000/60;
  s = String(minutes);
  line.replace("%1",s);
  oled.setTextXY(5,0); 
  oled.putString(line);
  
  // Line 6
  line = "COOK %1 WARM %2";  
  s = (digitalRead(cookModeSSRPin) == HIGH) ? "ON" : "OFF";
  line.replace("%1", s);
  s = (digitalRead(warmModeSSRPin) == HIGH) ? "ON" : "OFF";
  line.replace("%2", s);
  oled.setTextXY(7,0); 
  oled.putString(line);
}

// publish data over the serial link, to make it available for reporting

void publish(){
  Serial.print(currentTemperature, 1);                // 1 = one digit after the period
  Serial.print(';');                                  // just a delimiter, to help the receiver to parse it easily
  Serial.print(targetedTemperature, 1);               // 1 = one digit after the period
  Serial.print(';');                                  // just a delimiter, to help the receiver to parse it easily
  Serial.print(currentHumidity, 0);                   // 1 = no digit after the period
  Serial.print(';');                                  // just a delimiter, to help the receiver to parse it easily
  Serial.print(targetedHumidity, 0);                  // 1 = no digit after the period
  Serial.println();  
}
